﻿using Kivii.Linq;
using Kivii.Samples.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Security.Policy;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Samples.Transforms
{
    #region CRUDQ
    //[RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleQuery : RestfulQuery<Sample>
    {
        public List<Guid> BizKvids { get; set; }

        public bool IncludeLocation { get; set; }
        public bool IncludePackage { get; set; }
        public bool IncludeRoutes { get; set; }

        public override bool OnPreRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns, ref Expression<Func<Sample, bool>> where)
        {
            if (!BizKvids.IsNullOrEmpty()) where = o => Sql.In(o.BizKvid, BizKvids);
            return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
        }
        public override bool OnPostRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns)
        {
            if (IncludeLocation)
            {
                foreach (var item in rtns.Results)
                {
                    item.CurrentLocation = item.GetCurrentLocation();
                }
            }
            if (IncludePackage)
            {
                foreach (var item in rtns.Results)
                {
                    item.CurrentPackage = item.GetCurrentPackage();
                }
            }
            if (IncludeRoutes)
            {
                foreach (var item in rtns.Results)
                {
                    item.Routes = item.GetRoutes();
                }
            }
            return base.OnPostRestfulQuery(req, res, dbConnection, rtns);
        }
    }
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleCreate : RestfulCreate<Sample>
    {
        public override bool OnPreRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<Sample> rtns)
        {
            if (Item.Type != SampleType.Package) throw new Exception("只能新建包裹类型样品!");
            Item.Category = SampleType.Package.ToString();
            return base.OnPreRestfulCreate(req, res, dbConnection, rtns);
        }
    }
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleUpdate : RestfulUpdate<Sample>
    {
    }
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleRead : RestfulRead<Sample>
    {
        public bool IncludeLocation { get; set; }
        public bool IncludeChildrens { get; set; }
        public bool IncludeRoutes { get; set; }
        public bool IncludePackage { get; set; }
        public override bool OnPostRestfulRead(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulReadResponse<Sample> rtns)
        {
            if(rtns.Result== null) return base.OnPostRestfulRead(req, res, dbConnection, rtns);
            if (IncludeLocation)
            {
                rtns.Result.CurrentLocation = rtns.Result.GetCurrentLocation();
            }
            if (IncludeChildrens)
            {
                rtns.Result.Childrens = rtns.Result.GetChildrens();
            }
            if (IncludeRoutes)
            {
                rtns.Result.Routes = rtns.Result.GetRoutes();
            }
            if (IncludePackage)
            {
                rtns.Result.CurrentPackage = rtns.Result.GetCurrentPackage();
            }
            return base.OnPostRestfulRead(req, res, dbConnection, rtns);
        }

    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleDelete : RestfulDelete<Sample>
    {
    }
    #endregion

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "生成包裹")]
    public class SamplePackageInit : RestfulExecution<Sample>
    {
        public bool AutoCreate { get; set; }
        public string PackageName { get; set; }
        public bool IncludeLocation { get; set; }
        public bool IncludeRoutes { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            PackageName.ThrowIfNullOrEmpty("请输入包裹名称以查询！");
            var rtns = new RestfulReadResponse<Sample>();
            var conn = KiviiContext.GetOpenedDbConnection<Sample>();
            var exist = conn.Single<Sample>(o => o.Name == PackageName && o.Type == SampleType.Package);
            if (exist != null)
            {
                rtns.Result = exist;
            }
            else if (AutoCreate)
            {
                var item = new Sample();
                item.Name = PackageName;
                item.Type = SampleType.Package;
                item.OperateTime = DateTime.Now;
                item.Category = item.Type.ToString();
                conn.Insert(item);
                item.RemoveAllOnlyProperties();
                rtns.Result = item;
            }
            if (rtns.Result != null)
            {
                if (IncludeLocation)
                {
                    rtns.Result.CurrentLocation = rtns.Result.GetCurrentLocation();

                }
                if (IncludeRoutes)
                {
                    rtns.Result.Routes = rtns.Result.GetRoutes();

                }
            }
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description ="生成样品")]
    public class SampleAccept : RestfulExecution<Sample>
    {
        public Sample Item { get; set; }//要创建的样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public Route Route { get; set; }//自定义路由信息,可不填写

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null).ThrowIfTrue("缺少要创建的样品实例!");
            (Item.Name.IsNullOrEmpty()).ThrowIfTrue("缺少样品名称信息!");
            (Location == null).ThrowIfTrue("缺少目的地点配置信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少目的地点配置InternalCode信息");
            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local= conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}目的地点配置信息!");
            
            var rtns = new RestfulCreateResponse<Sample>();
            rtns.Results = new List<Sample>();

            var accept = Item.Accept(local, Route);
            rtns.Results.Add(accept);
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "样品拆分")]
    public class SampleSplit:RestfulExecution<Sample>
    {
        public Guid Kvid { get; set; }//要拆分的样品Kvid

        public List<Sample> Samples { get; set; }//拆分出的子样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty).ThrowIfTrue("缺少要拆分的样品Kvid");
            Samples.ThrowIfNullOrEmpty("缺少拆分的样品信息");
            (Location == null).ThrowIfTrue("缺少目的地点配置信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少目的地点配置InternalCode信息");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}目的地点配置信息!");

            var sample = new Sample();
            sample.Kvid = Kvid;

            var splited = sample.Split(Samples, local);
            var rtns = new RestfulCreateResponse<Sample>();
            rtns.Results = new List<Sample>();
            rtns.Results.AddRange(splited);

            return rtns;
        }
    }

    //[RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "样品流转")]
    public class SampleAssign : RestfulExecution<Sample>
    {
        public Guid Kvid { get; set; }//要流转样品信息
        public List<Guid> Kvids { get; set; }//要流转样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public Route Route { get; set; }//自定义路由信息,可不填写

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()).ThrowIfTrue("缺少要流转的样品信息!");
            Location.ThrowIfNull("缺少目的地点信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少目的地点配置InternalCode信息");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}目的地点配置信息!");

            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);

            var rtns = new RestfulUpdateResponse<Sample>();
            rtns.Results = new List<Sample>();

            foreach (var kvid in Kvids)
            {
                var sample = new Sample();
                sample.Kvid = kvid;

                var assign = sample.Assign(local, Route);
                rtns.Results.Add(assign);
            }
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "样品入库")]
    public class SampleInbound : RestfulExecution<Sample>
    {
        public Sample Sample { get; set; }//要流转样品信息
        public List<Sample> Samples { get; set; }//要流转样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public Route Route { get; set; }//自定义路由信息,可不填写

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Sample == null && Samples.IsNullOrEmpty()).ThrowIfTrue("缺少要入库的样品信息!");
            Location.ThrowIfNull("缺少目的地点信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少目的地点配置InternalCode信息");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}目的地点配置信息!");

            if (Samples.IsNullOrEmpty()) Samples = new List<Sample>();
            if (Sample != null) Samples.Add(Sample);

            var rtns = new RestfulExecutionResponse<Sample>();
            rtns.Results = new List<Sample>();

            foreach (var sample in Samples)
            {
                var assign = sample.Inbound(local, Route);
                rtns.Results.Add(assign);
            }
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "包裹清空")]
    public class SamplePackageClear : RestfulExecution<Sample>
    {
        public Guid PackageKvid { get; set; }//要流转样品信息
        public List<Guid> PackageKvids { get; set; }//要流转样品信息
        public Location Location { get; set; }//地点位置信息,只需要InternalCode
        public Route Route { get; set; }//自定义路由信息,可不填写
        public override object OnExecution(IRequest req, IResponse res)
        {
            (PackageKvid == Guid.Empty && PackageKvids.IsNullOrEmpty()).ThrowIfTrue("缺少要打包的样品信息!");
            Location.ThrowIfNull("缺少打包地点信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少打包地点配置InternalCode信息");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}打包地点配置信息!");

            if (PackageKvids.IsNullOrEmpty()) PackageKvids = new List<Guid>();
            if (PackageKvid != Guid.Empty) PackageKvids.Add(PackageKvid);

            var packages = new List<Sample>();
            foreach (var kvid in PackageKvids)
            {
                var package = new Sample();
                package.Kvid = kvid;
                packages.Add(package);
            }
            var rtns = new RestfulExecutionResponse<Sample>();
            rtns.Results = new List<Sample>();

            var trans = conn.OpenTransaction();
            try
            {
                rtns.Results = packages.PackageClear(local, Route, conn);
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "包裹清空查询")]
    public class SamplePackageClearQuery : RestfulExecution<Sample>
    {
        public DateTime DealTime { get; set; }
        public DateTime DeadTime { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Sample>();
            var queryPackageKvids = conn.From<Sample>();
            queryPackageKvids.Where(o => o.PackageKvid != Guid.Empty);
            if (DealTime != DateTime.MinValue) queryPackageKvids.And(o => o.DealTime < DealTime);
            if (DeadTime != DateTime.MinValue) queryPackageKvids.And(o => o.DeadTime < DeadTime);
            queryPackageKvids.GroupBy(o => o.PackageKvid);
            queryPackageKvids.Select(o => o.PackageKvid);

            var queryPackages = conn.From<Sample>();
            queryPackages.Where(o => o.Status == (int)PackageStatus.Inbound && Sql.In(o.Kvid, queryPackageKvids));
            var packages = conn.Select(queryPackages);

            var rtns = new RestfulQueryResponse<Sample>();
            rtns.Results = new List<Sample>();
            rtns.Results.AddRange(packages);
            rtns.Total = rtns.Results.Count;

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "打包样查找")]
    public class SamplePrePackaging : RestfulExecution<Sample>
    {
        public Guid OwnerKvid { get; set; }//要打包的样品的所属信息

        public string BizId { get; set; }//传入样品的业务BizId或BiziKvid或者对应Rfid的Tid号 结合OwnerKvid查找对应的样品,包含查询子样品

        public Guid BizKvid { get; set; }//三者不能都为空

        public string Tid { get; set; }

        public bool IncludeLocation { get; set; }
        public bool IncludeChildrens { get; set; }
        public bool IncludeRoutes { get; set; }
        public bool IncludePackage { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (OwnerKvid == Guid.Empty) OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            if (BizId.IsNullOrEmpty() && Tid.IsNullOrEmpty() && BizKvid == Guid.Empty) throw new Exception("请传入要查询的样品业务信息!");
            var conn = KiviiContext.GetOpenedDbConnection<Sample>();
            Sample rootSample = null;
            //Guid sampleRootKvid = Guid.Empty;
            if (BizKvid != Guid.Empty)
            {
                var queryRootSample = conn.From<Sample>();
                queryRootSample.Where(o => o.BizKvid == BizKvid);
                rootSample = conn.Single(queryRootSample);
                if (rootSample != null)
                {
                    BizId = rootSample.BizId;
                }
            }

            if (rootSample == null)
            {
                if (!Tid.IsNullOrEmpty())
                {
                    var tidKvid = Tid.GetKvidByTid();
                    if (tidKvid != Guid.Empty)
                    {
                        var queryRootSample = conn.From<Sample>();
                        queryRootSample.Where(o => o.BizKvid == tidKvid);
                        rootSample = conn.Single(queryRootSample);
                        if (rootSample != null)
                        {
                            BizId = rootSample.BizId;
                        }
                    }

                }
                else if (!BizId.IsNullOrEmpty())
                {
                    var queryRootSample = conn.From<Sample>();
                    queryRootSample.Where(o => o.BizId == BizId);
                    queryRootSample.OrderBy(o => o.CreateTime);
                    rootSample = conn.Single(queryRootSample);
                    //var querySampleRootKvid = conn.From<Sample>();
                    //querySampleRootKvid.Where(o => o.BizId == BizId);
                    //querySampleRootKvid.Select(o => o.RootKvid);
                    //sampleRootKvid = conn.Single<Guid>(querySampleRootKvid);
                }
            }
            if (rootSample == null) throw new Exception("未找到指定样品信息!");

            var rtns = new RestfulExecutionResponse<Sample>();
            rtns.Results = new List<Sample>();

            var querySample = conn.From<Sample>();
            querySample.Where(o => o.OwnerKvid == OwnerKvid && o.RootKvid == rootSample.RootKvid && o.BizId == BizId && o.Type == SampleType.Node);//只查子样品
            var sample = conn.Single(querySample);
            if (sample == null)
            {
                sample = new Sample();
                sample.PopulateWith(rootSample);
            }
            if (IncludeLocation)
            {
                sample.CurrentLocation = sample.GetCurrentLocation();
            }
            if (IncludeChildrens)
            {
                sample.Childrens = sample.GetChildrens();
            }
            if (IncludeRoutes)
            {
                sample.Routes = sample.GetRoutes();
            }
            if (IncludePackage)
            {
                sample.CurrentPackage = sample.GetCurrentPackage();
            }
            if (sample != null) rtns.Results.Add(sample);

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "已打包样查找")]
    public class SamplePackaged : RestfulExecution<Sample>
    {
        public string BizId { get; set; }//传入样品的业务BizId或BiziKvid或者对应Rfid的Tid号 结合OwnerKvid查找对应的样品,包含查询子样品

        public Guid BizKvid { get; set; }//三者不能都为空

        public string Tid { get; set; }

        public bool IncludeLocation { get; set; }
        public bool IncludeChildrens { get; set; }
        public bool IncludeRoutes { get; set; }
        public bool IncludePackage { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (BizId.IsNullOrEmpty() && Tid.IsNullOrEmpty() && BizKvid == Guid.Empty) throw new Exception("请传入要查询的样品业务信息!");
            var conn = KiviiContext.GetOpenedDbConnection<Sample>();
            Sample rootSample = null;
            //Guid sampleRootKvid = Guid.Empty;
            if (BizKvid != Guid.Empty)
            {
                var queryRootSample = conn.From<Sample>();
                queryRootSample.Where(o => o.BizKvid == BizKvid);
                rootSample = conn.Single(queryRootSample);
                if (rootSample != null)
                {
                    BizId = rootSample.BizId;
                }
            }

            if (rootSample == null)
            {
                if (!Tid.IsNullOrEmpty())
                {
                    var tidKvid = Tid.GetKvidByTid();
                    if (tidKvid != Guid.Empty)
                    {
                        var queryRootSample = conn.From<Sample>();
                        queryRootSample.Where(o => o.BizKvid == tidKvid);
                        rootSample = conn.Single(queryRootSample);
                        if (rootSample != null)
                        {
                            BizId = rootSample.BizId;
                        }
                    }

                }
                else if (!BizId.IsNullOrEmpty())
                {
                    var queryRootSample = conn.From<Sample>();
                    queryRootSample.Where(o => o.BizId == BizId);
                    rootSample = conn.Single(queryRootSample);
                    //var querySampleRootKvid = conn.From<Sample>();
                    //querySampleRootKvid.Where(o => o.BizId == BizId);
                    //querySampleRootKvid.Select(o => o.RootKvid);
                    //sampleRootKvid = conn.Single<Guid>(querySampleRootKvid);
                }
            }
            if (rootSample == null) throw new Exception("未找到指定样品信息!");

            var rtns = new RestfulExecutionResponse<Sample>();
            rtns.Results = new List<Sample>();

            var querySample = conn.From<Sample>();
            querySample.Where(o => o.PackageKvid != Guid.Empty && o.RootKvid == rootSample.RootKvid && o.BizId == BizId);
            var sample = conn.Single(querySample);
            if (sample != null)
            {
                if (IncludeLocation)
                {
                    sample.CurrentLocation = sample.GetCurrentLocation();
                }
                if (IncludeChildrens)
                {
                    sample.Childrens = sample.GetChildrens();
                }
                if (IncludeRoutes)
                {
                    sample.Routes = sample.GetRoutes();
                }
                if (IncludePackage)
                {
                    sample.CurrentPackage = sample.GetCurrentPackage();
                }
                rtns.Results.Add(sample);
            }
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "样品打包")]
    public class SamplePackaging : RestfulExecution<Sample>
    {
        public Guid Kvid { get; set; }//要流转样品信息
        public List<Guid> Kvids { get; set; }//要流转样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public Route Route { get; set; }//自定义路由信息,可不填写

        public Sample Package { get; set; }//指定包裹信息,Package.PackageName可传入包裹的区域信息,例如包裹A0001中的1号袋子
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()).ThrowIfTrue("缺少要打包的样品信息!");
            Location.ThrowIfNull("缺少打包地点信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少打包地点配置InternalCode信息");
            Package.ThrowIfNull("缺少包裹信息!");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}打包地点配置信息!");
            Sample package = null;
            if (Package.Kvid != Guid.Empty)
            {
                package = conn.SingleById<Sample>(Package.Kvid);
                if (package != null)
                {
                    package.PackageName = Package.PackageName;
                    Package = package;
                }
            }

            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);
            var samples = new List<Sample>();
            foreach (var kvid in Kvids)
            {
                var sample = new Sample();
                sample.Kvid = kvid;
                samples.Add(sample);
            }

            var rtns = new RestfulExecutionResponse<Sample>();
            rtns.Results = new List<Sample>();

            var trans = conn.OpenTransaction();
            try
            {
                rtns.Results = samples.Packaging(package, local, Route, conn);
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "包裹移库，已经打包的样品迁移到其他包裹中")]
    public class SamplePackageMove : RestfulExecution<Sample>
    {
        public Guid SampleKvid { get; set; }//要迁移的样品
        public List<Guid> SampleKvids { get; set; }//要迁移的样品（此样品前提是已经打包的否则不允许移库）

        public Guid PackageKvid { get; set; }//目标包裹信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public Route Route { get; set; }//自定义路由信息,可不填写
        public override object OnExecution(IRequest req, IResponse res)
        {
            (SampleKvid == Guid.Empty && SampleKvids.IsNullOrEmpty()).ThrowIfTrue("缺少要打包的样品信息!");
            Location.ThrowIfNull("缺少打包地点信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少打包地点配置InternalCode信息");
            PackageKvid.ThrowIfNotEmpty("缺少包裹信息!");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}打包地点配置信息!");
            Sample package = conn.SingleById<Sample>(PackageKvid);

            if (SampleKvids.IsNullOrEmpty()) SampleKvids = new List<Guid>();
            if (SampleKvid != Guid.Empty) SampleKvids.Add(SampleKvid);
            var samples = new List<Sample>();
            foreach (var kvid in SampleKvids)
            {
                var sample = new Sample();
                sample.Kvid = kvid;
                samples.Add(sample);
            }
            var rtns = new RestfulExecutionResponse<Sample>();
            rtns.Results = new List<Sample>();

            var trans = conn.OpenTransaction();
            try
            {
                rtns.Results = samples.PackageMoving(package, local, Route, conn);
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "包裹取样")]
    public class SampleUnPackaging : RestfulExecution<Sample>
    {
        public Guid Kvid { get; set; }//要取样品的信息
        public List<Guid> Kvids { get; set; }//要取样品的信息
        //public Guid Kvid { get; set; }//要取样品的信息
        public Location Location { get; set; }//地点位置信息,只需要InternalCode
        public Route Route { get; set; }//自定义路由信息,可不填写

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()).ThrowIfTrue("缺少要取样品的信息!");
            //(Kvid == Guid.Empty).ThrowIfTrue("缺少要要取样品信息!");
            Location.ThrowIfNull("缺少取样地点信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少取样地点配置InternalCode信息");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}打包地点配置信息!");
            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);
            List<Sample> samples = new List<Sample>();
            foreach (var kvid in Kvids)
            {
                var sample = new Sample();
                sample.Kvid = kvid;
                samples.Add(sample);
            }

            var rtns = new RestfulExecutionResponse<Sample>();
            rtns.Results = new List<Sample>();

            var trans = conn.OpenTransaction();
            try
            {
                foreach (var item in samples)
                {
                    var result = item.UnPackaging(local, Route, conn);
                    rtns.Results.Add(result);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    //[RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "所在地样品")]
    public class SampleLocationQuery : RestfulQuery<Sample>
    {
        public Guid LocationKvid { get; set; }
        public string LocationInternalCode { get; set; }

        private Location location;

        public override bool OnPreRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns, ref Expression<Func<Sample, bool>> where)
        {
            (LocationKvid == Guid.Empty && LocationInternalCode.IsNullOrEmpty()).ThrowIfTrue("请传入要查询的地点信息!");
            //Location location = null;
            if (LocationKvid != Guid.Empty) location = dbConnection.SingleById<Location>(LocationKvid);
            if (location == null)
            {
                if (!LocationInternalCode.IsNullOrEmpty()) location = dbConnection.Single<Location>(o => o.InternalCode == LocationInternalCode);
                location.ThrowIfNull("未找到指定的地点信息!");
            }
            var queryRoutes = dbConnection.From<Route>();
            queryRoutes.Where(o => o.CurrentLocationKvid == location.Kvid && o.NextLocationKvid == Guid.Empty);
            queryRoutes.Select(o => o.SampleKvid);
            where = o => Sql.In(o.Kvid, queryRoutes);
            return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
        }
        public override object OnRestfulQueryResponse(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns)
        {
            foreach (var item in rtns.Results)
            {
                item.CurrentLocation = location;
            }
            return base.OnRestfulQueryResponse(req, res, dbConnection, rtns);
        }
    }

}
